@@ -36,17 +36,11 @@ static int state_lock_(const char *file, int lock, int timeout)
 	struct flock lck;
 	struct stat st;
 	char lcktxt[12];
-	char *nfile;
+	char *nfile = lockfile;
 
-	if (!do_lock)
+	if (do_lock <= 0)
 		return 0;
-	nfile = malloc(strlen(file) + 6);
-	if (nfile == NULL) {
-		error("No enough memory...");
-		return -ENOMEM;
-	}
-	strcpy(nfile, file);
-	strcat(nfile, ".lock");
+
 	lck.l_type = lock ? F_WRLCK : F_UNLCK;
 	lck.l_whence = SEEK_SET;
 	lck.l_start = 0;
@@ -107,7 +101,6 @@ static int state_lock_(const char *file, int lock, int timeout)
 		goto out;
 	}
 out:
-	free(nfile);
 	return err;
 }
 
