@@ -63,7 +63,7 @@ static void error(const char *format, ...)
 
 static void usage(void)
 {
-	fprintf(stderr,
+	printf(
 		"Usage: amidi options\n"
 		"\n"
 		"-h, --help             this help\n"
@@ -82,7 +82,7 @@ static void usage(void)
 
 static void version(void)
 {
-	fputs("amidi version " SND_UTIL_VERSION_STR "\n", stderr);
+	puts("amidi version " SND_UTIL_VERSION_STR);
 }
 
 static void *my_malloc(size_t size)
@@ -100,53 +100,58 @@ static void list_device(snd_ctl_t *ctl, int card, int device)
 	snd_rawmidi_info_t *info;
 	const char *name;
 	const char *sub_name;
-	int subs;
+	int subs, subs_in, subs_out;
+	int sub;
 	int err;
 
 	snd_rawmidi_info_alloca(&info);
 	snd_rawmidi_info_set_device(info, device);
-	snd_rawmidi_info_set_subdevice(info, 0);
+
+	snd_rawmidi_info_set_stream(info, SND_RAWMIDI_STREAM_INPUT);
+	err = snd_ctl_rawmidi_info(ctl, info);
+	if (err >= 0)
+		subs_in = snd_rawmidi_info_get_subdevices_count(info);
+	else
+		subs_in = 0;
+
 	snd_rawmidi_info_set_stream(info, SND_RAWMIDI_STREAM_OUTPUT);
-	if ((err = snd_ctl_rawmidi_info(ctl, info)) < 0 &&
-	    err != -ENOENT) {
-		error("cannot get rawmidi information %d:%d: %s",
-		      card, device, snd_strerror(err));
+	err = snd_ctl_rawmidi_info(ctl, info);
+	if (err >= 0)
+		subs_out = snd_rawmidi_info_get_subdevices_count(info);
+	else
+		subs_out = 0;
+
+	subs = subs_in > subs_out ? subs_in : subs_out;
+	if (!subs)
 		return;
-	}
-	if (err == -ENOENT) {
-		snd_rawmidi_info_set_stream(info, SND_RAWMIDI_STREAM_INPUT);
-		if ((err = snd_ctl_rawmidi_info(ctl, info)) < 0 &&
-		    err != -ENOENT) {
-			error("cannot get rawmidi information %d:%d: %s",
-			      card, device, snd_strerror(err));
+
+	for (sub = 0; sub < subs; ++sub) {
+		snd_rawmidi_info_set_stream(info, sub < subs_in ?
+					    SND_RAWMIDI_STREAM_INPUT :
+					    SND_RAWMIDI_STREAM_OUTPUT);
+		snd_rawmidi_info_set_subdevice(info, sub);
+		err = snd_ctl_rawmidi_info(ctl, info);
+		if (err < 0) {
+			error("cannot get rawmidi information %d:%d:%d: %s\n",
+			      card, device, sub, snd_strerror(err));
 			return;
 		}
-	}
-	if (err == -ENOENT)
-		return;
-	name = snd_rawmidi_info_get_name(info);
-	sub_name = snd_rawmidi_info_get_subdevice_name(info);
-	subs = snd_rawmidi_info_get_subdevices_count(info);
-	if (sub_name[0] == '\0') {
-		if (subs == 1)
-			printf("hw:%d,%d    %s\n", card, device, name);
-		else
-			printf("hw:%d,%d    %s (%d subdevices)\n",
-			       card, device, name, subs);
-	} else {
-		int sub = 0;
-		for (;;) {
-			printf("hw:%d,%d,%d  %s\n",
+		name = snd_rawmidi_info_get_name(info);
+		sub_name = snd_rawmidi_info_get_subdevice_name(info);
+		if (sub == 0 && sub_name[0] == '\0') {
+			printf("%c%c  hw:%d,%d    %s",
+			       sub < subs_in ? 'I' : ' ',
+			       sub < subs_out ? 'O' : ' ',
+			       card, device, name);
+			if (subs > 1)
+				printf(" (%d subdevices)", subs);
+			putchar('\n');
+			break;
+		} else {
+			printf("%c%c  hw:%d,%d,%d  %s\n",
+			       sub < subs_in ? 'I' : ' ',
+			       sub < subs_out ? 'O' : ' ',
 			       card, device, sub, sub_name);
-			if (++sub >= subs)
-				break;
-			snd_rawmidi_info_set_subdevice(info, sub);
-			if ((err = snd_ctl_rawmidi_info(ctl, info)) < 0) {
-				error("cannot get rawmidi information %d:%d:%d: %s",
-				      card, device, sub, snd_strerror(err));
-				break;
-			}
-			sub_name = snd_rawmidi_info_get_subdevice_name(info);
 		}
 	}
 }
@@ -189,7 +194,7 @@ static void device_list(void)
 		error("no sound card found");
 		return;
 	}
-	puts("Device    Name");
+	puts("Dir Device    Name");
 	do {
 		list_card_devices(card);
 		if ((err = snd_card_next(&card)) < 0) {
@@ -381,7 +386,7 @@ static void sig_handler(int dummy)
 	stop = 1;
 }
 
-void add_send_hex_data(const char *str)
+static void add_send_hex_data(const char *str)
 {
 	int length;
 	char *s;
@@ -401,8 +406,8 @@ void add_send_hex_data(const char *str)
 
 int main(int argc, char *argv[])
 {
-	static char short_options[] = "hVlLp:s:r:S::dt:a";
-	static struct option long_options[] = {
+	static const char short_options[] = "hVlLp:s:r:S::dt:a";
+	static const struct option long_options[] = {
 		{"help", 0, NULL, 'h'},
 		{"version", 0, NULL, 'V'},
 		{"list-devices", 0, NULL, 'l'},
@@ -520,7 +525,7 @@ int main(int argc, char *argv[])
 	else
 		outputp = NULL;
 
-	if ((err = snd_rawmidi_open(inputp, outputp, port_name, 0)) < 0) {
+	if ((err = snd_rawmidi_open(inputp, outputp, port_name, SND_RAWMIDI_NONBLOCK)) < 0) {
 		error("cannot open port \"%s\": %s", port_name, snd_strerror(err));
 		goto _exit2;
 	}
@@ -528,10 +533,15 @@ int main(int argc, char *argv[])
 	if (inputp)
 		snd_rawmidi_read(input, NULL, 0); /* trigger reading */
 
-	if (send_data &&
-	    ((err = snd_rawmidi_write(output, send_data, send_data_length))) < 0) {
-		error("cannot send data: %s", snd_strerror(err));
-		goto _exit;
+	if (send_data) {
+		if ((err = snd_rawmidi_nonblock(output, 0)) < 0) {
+			error("cannot set blocking mode: %s", snd_strerror(err));
+			goto _exit;
+		}
+		if ((err = snd_rawmidi_write(output, send_data, send_data_length)) < 0) {
+			error("cannot send data: %s", snd_strerror(err));
+			goto _exit;
+		}
 	}
 
 	if (inputp) {
@@ -540,7 +550,6 @@ int main(int argc, char *argv[])
 		struct pollfd *pfds;
 
 		timeout *= 1000;
-		snd_rawmidi_nonblock(input, 1);
 		npfds = snd_rawmidi_poll_descriptors_count(input);
 		pfds = alloca(npfds * sizeof(struct pollfd));
 		snd_rawmidi_poll_descriptors(input, pfds, npfds);
